"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
/* eslint-disable no-console */
const { homebridge } = window;
const $newTokenButton = document.getElementById('winix-new-token');
const $linkAccountHeader = document.getElementById('winix-link-account-header');
// Register click handler for the "Link Account" button
$newTokenButton.addEventListener('click', () => showLoginForm());
function renderForm() {
    var _a;
    return __awaiter(this, void 0, void 0, function* () {
        // Hide initial loading spinner
        homebridge.hideSpinner();
        const [config] = yield homebridge.getPluginConfig();
        const hasToken = (_a = config === null || config === void 0 ? void 0 : config.auth) === null || _a === void 0 ? void 0 : _a.refreshToken;
        if (hasToken) {
            showConfigForm();
        }
        else {
            yield showLoginForm();
        }
    });
}
// Init
renderForm();
function showLoginForm() {
    return __awaiter(this, void 0, void 0, function* () {
        // Hide the standard form
        homebridge.hideSchemaForm();
        $newTokenButton === null || $newTokenButton === void 0 ? void 0 : $newTokenButton.style.setProperty('display', 'none');
        $linkAccountHeader === null || $linkAccountHeader === void 0 ? void 0 : $linkAccountHeader.style.setProperty('display', 'block');
        const loginForm = homebridge.createForm({
            schema: {
                type: 'object',
                properties: {
                    email: {
                        title: 'Email',
                        type: 'string',
                        'x-schema-form': {
                            type: 'email',
                        },
                        required: true,
                    },
                    password: {
                        title: 'Password',
                        type: 'string',
                        'x-schema-form': {
                            type: 'password',
                        },
                        required: true,
                    },
                },
            },
        }, {}, 'Log In', (yield hasExistingAuth()) ? 'Back' : undefined);
        loginForm.onSubmit(({ email, password }) => __awaiter(this, void 0, void 0, function* () {
            homebridge.showSpinner();
            try {
                const auth = yield homebridge.request('/login', { email, password });
                yield setExistingAuth(auth);
                showConfigForm();
            }
            catch (e) {
                const error = e;
                console.error('error logging in', error.message);
                homebridge.toast.error(error.message, 'Winix Login Failed');
            }
            finally {
                homebridge.hideSpinner();
            }
        }));
        // We know already that there is existing auth since the 'Back' button is shown,
        // so we can just go back to the config form
        loginForm.onCancel(() => showConfigForm());
    });
}
function showConfigForm() {
    $newTokenButton === null || $newTokenButton === void 0 ? void 0 : $newTokenButton.style.setProperty('display', 'block');
    $linkAccountHeader === null || $linkAccountHeader === void 0 ? void 0 : $linkAccountHeader.style.setProperty('display', 'none');
    homebridge.showSchemaForm();
}
function setExistingAuth(auth) {
    return __awaiter(this, void 0, void 0, function* () {
        const [config, ...otherConfigs] = yield homebridge.getPluginConfig();
        yield homebridge.updatePluginConfig([
            Object.assign(Object.assign({}, config), { auth }),
            ...otherConfigs,
        ]);
        yield homebridge.savePluginConfig();
        homebridge.toast.success('Refresh Token Updated', 'Winix Login Successful');
    });
}
function hasExistingAuth() {
    var _a;
    return __awaiter(this, void 0, void 0, function* () {
        const [config] = yield homebridge.getPluginConfig();
        return !!((_a = config === null || config === void 0 ? void 0 : config.auth) === null || _a === void 0 ? void 0 : _a.refreshToken);
    });
}
